# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .. import decorators
from .. import enums
from ..protocols import swd
from .. import registers

import collections
import time


Identity = collections.namedtuple('Identity', ['version_code', 'part_no'])

UNLOCK_METHODS = {}


class KinetisException(Exception):
    """Exception generated when polling fails."""
    pass


def unlock_kinetis_identified(identity, flags):
    """Checks whether the given flags are a valid identity.

    Args:
      identity (Identity): the identity to validate against
      flags (register.IDCodeRegisterFlags): the set idcode flags

    Returns:
      ``True`` if the given ``flags`` correctly identify the the debug
      interface, otherwise ``False``.
    """
    if flags.version_code != identity.version_code:
        return False

    if flags.part_no != identity.part_no:
        return False

    return flags.valid


def unlock_kinetis_abort_clear():
    """Returns the abort register clear code.

    Returns:
      The abort register clear code.
    """
    flags = registers.AbortRegisterFlags()
    flags.STKCMPCLR = 1
    flags.STKERRCLR = 1
    flags.WDERRCLR = 1
    flags.ORUNERRCLR = 1
    return flags.value


def unlock_kinetis_read_until_ack(jlink, address):
    """Polls the device until the request is acknowledged.

    Sends a read request to the connected device to read the register at the
    given 'address'.  Polls indefinitely until either the request is ACK'd or
    the request ends in a fault.

    Args:
      jlink (JLink): the connected J-Link
      address (int) the address of the register to poll

    Returns:
      ``SWDResponse`` object on success.

    Raises:
      KinetisException: when read exits with non-ack or non-wait status.

    Note:
      This function is required in order to avoid reading corrupt or otherwise
      invalid data from registers when communicating over SWD.
    """
    request = swd.ReadRequest(address, ap=True)
    response = None
    while True:
        response = request.send(jlink)
        if response.ack():
            break
        elif response.wait():
            continue
        raise KinetisException('Read exited with status: %s', response.status)

    return response


def unlock_kinetis_swd(jlink):
    """Unlocks a Kinetis device over SWD.

    Steps Involved in Unlocking:
      1.  Verify that the device is configured to read/write from the CoreSight
          registers; this is done by reading the Identification Code Register
          and checking its validity.  This register is always at 0x0 on reads.
      2.  Check for errors in the status register.  If there are any errors,
          they must be cleared by writing to the Abort Register (this is always
          0x0 on writes).
      3.  Turn on the device power and debug power so that the target is
          powered by the J-Link as more power is required during an unlock.
      4.  Assert the ``RESET`` pin to force the target to hold in a reset-state
          as to avoid interrupts and other potentially breaking behaviour.
      5.  At this point, SWD is configured, so send a request to clear the
          errors, if any, that may currently be set.
      6.  Our next SWD request selects the MDM-AP register so that we can start
          sending unlock instructions.  ``SELECT[31:24] = 0x01`` selects it.
      7.  Poll the MDM-AP Status Register (AP[1] bank 0, register 0) until the
          flash ready bit is set to indicate we can flash.
      8.  Write to the MDM-AP Control Register (AP[1] bank 0, register 1) to
          request a flash mass erase.
      9.  Poll the system until the flash mass erase bit is acknowledged in the
          MDM-AP Status Register.
      10. Poll the control register until it clears it's mass erase bit to
          indicate that it finished mass erasing, and therefore the system is
          now unsecure.

    Args:
      jlink (JLink): the connected J-Link

    Returns:
      ``True`` if the device was unlocked successfully, otherwise ``False``.

    Raises:
      KinetisException: when the device cannot be unlocked or fails to unlock.

    See Also:
      `NXP Forum <https://community.nxp.com/thread/317167>`_.

    See Also:
      `Kinetis Docs <nxp.com/files/32bit/doc/ref_manual/K12P48M50SF4RM.pdf>`
    """
    SWDIdentity = Identity(0x2, 0xBA01)
    jlink.power_on()
    jlink.coresight_configure()

    # 1. Verify that the device is configured properly.
    flags = registers.IDCodeRegisterFlags()
    flags.value = jlink.coresight_read(0x0, False)
    if not unlock_kinetis_identified(SWDIdentity, flags):
        return False

    # 2. Check for errors.
    flags = registers.ControlStatusRegisterFlags()
    flags.value = jlink.coresight_read(0x01, False)
    if flags.STICKYORUN or flags.STICKYCMP or flags.STICKYERR or flags.WDATAERR:
        jlink.coresight_write(0x0, unlock_kinetis_abort_clear(), False)

    # 3. Turn on device power and debug.
    flags = registers.ControlStatusRegisterFlags()
    flags.value = 0
    flags.CSYSPWRUPREQ = 1  # System power-up request
    flags.CDBGPWRUPREQ = 1  # Debug power-up request
    jlink.coresight_write(0x01, flags.value, False)

    # 4. Assert the reset pin.
    jlink.set_reset_pin_low()
    time.sleep(1)

    # 5. Send a SWD Request to clear any errors.
    request = swd.WriteRequest(0x0, False, unlock_kinetis_abort_clear())
    request.send(jlink)

    # 6. Send a SWD Request to select the MDM-AP register, SELECT[31:24] = 0x01
    request = swd.WriteRequest(0x2, False, (1 << 24))
    request.send(jlink)

    try:
        # 7. Poll until the Flash-ready bit is set in the status register flags.
        #    Have to read first to ensure the data is valid.
        unlock_kinetis_read_until_ack(jlink, 0x0)
        flags = registers.MDMAPStatusRegisterFlags()
        flags.flash_ready = 0
        while not flags.flash_ready:
            flags.value = unlock_kinetis_read_until_ack(jlink, 0x0).data

        # 8. System may still be secure at this point, so request a mass erase.
        #    AP[1] bank 0, register 1 is the MDM-AP Control Register.
        flags = registers.MDMAPControlRegisterFlags()
        flags.flash_mass_erase = 1
        request = swd.WriteRequest(0x1, True, flags.value)
        request.send(jlink)

        # 9. Poll the status register until the mass erase command has been
        #    accepted.
        unlock_kinetis_read_until_ack(jlink, 0x0)
        flags = registers.MDMAPStatusRegisterFlags()
        flags.flash_mass_erase_ack = 0
        while not flags.flash_mass_erase_ack:
            flags.value = unlock_kinetis_read_until_ack(jlink, 0x0).data

        # 10. Poll the control register until the ``flash_mass_erase`` bit is
        #     cleared, which is done automatically when the mass erase
        #     finishes.
        unlock_kinetis_read_until_ack(jlink, 0x1)
        flags = registers.MDMAPControlRegisterFlags()
        flags.flash_mass_erase = 1
        while flags.flash_mass_erase:
            flags.value = unlock_kinetis_read_until_ack(jlink, 0x1).data

    except KinetisException as e:
        jlink.set_reset_pin_high()
        return False

    jlink.set_reset_pin_high()
    time.sleep(1)

    jlink.reset()

    return True


UNLOCK_METHODS[enums.JLinkInterfaces.SWD] = unlock_kinetis_swd


def unlock_kinetis_jtag(jlink):
    """Unlocks a Kinetis device over JTAG.

    Note:
      Currently not implemented.

    Args:
      jlink (JLink): the connected J-Link

    Returns:
      ``True`` if the device was unlocked successfully, otherwise ``False``.

    Raises:
      NotImplementedError: always.
    """
    JTAGIdentity = Identity(0x4, 0xBA0)
    raise NotImplementedError('Unlock Kinetis over JTAG is not implemented.')


UNLOCK_METHODS[enums.JLinkInterfaces.JTAG] = unlock_kinetis_jtag


@decorators.async_decorator
def unlock_kinetis(jlink):
    """Unlock for Freescale Kinetis K40 or K60 device.

    Args:
      jlink (JLink): an instance of a J-Link that is connected to a target.

    Returns:
      ``True`` if the device was successfully unlocked, otherwise ``False``.

    Raises:
      ValueError: if the J-Link is not connected to a target.
    """
    if not jlink.connected():
        raise ValueError('No target to unlock.')

    method = UNLOCK_METHODS.get(jlink.tif, None)
    if method is None:
        raise NotImplementedError('Unsupported target interface for unlock.')

    return method(jlink)


__all__ = ['unlock_kinetis']



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.threads as threads

import unittest


class TestThreads(unittest.TestCase):
    """Unit test for the `threads` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestThreads): the `TestThreads` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestThreads): the `TestThreads` instance

        Returns:
          `None`
        """
        pass

    def test_thread(self):
        """Tests that a thread can be created and joined for a return value.

        Args:
          self (TestThreads): the `TestThreads` instance

        Returns:
          `None`
        """
        def thread_func():
            return 4

        def thread_func_with_args(x, y):
            return (x + y)

        thread = threads.ThreadReturn(target=thread_func)
        thread.start()
        self.assertEqual(4, thread.join())

        thread = threads.ThreadReturn(target=thread_func_with_args, args=(2, 3))
        thread.start()
        self.assertEqual(5, thread.join())


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import os
import subprocess
import sys


def root_dir():
    """Retrives the root testing directory.

    Returns:
      The root testing directory.
    """
    dirname = os.path.abspath(os.path.dirname(__file__))
    return os.path.abspath(os.path.join(dirname, os.pardir))


def firmware_path(firmware):
    """Returns the path to given firmware, provided it exists.

    Args:
      firmware (str): the firmware to search for

    Returns:
      The file path to the firmware if it exists, otherwise ``None``.
    """
    fw = os.path.join(root_dir(), 'firmware', firmware, 'build', 'firmware.bin')
    if not os.path.isfile(fw):
        return None

    if not sys.platform.startswith('cygwin'):
        return fw

    cygpath_exe = os.path.join(os.path.abspath(os.sep), 'bin', 'cygpath.exe')
    cygpath_cmd = [cygpath_exe, '-a', '-m', '-f', '-']
    cygpath_proc = subprocess.Popen(cygpath_cmd,
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE)
    cygpath_proc.stdin.write(fw + os.sep)
    return cygpath_proc.stdout.readline().rstrip()


def flash(jlink, firmware):
    """Flashes the given firmware to the target.

    Args:
      jlink (JLink): the connected ``JLink`` instance
      firmware (str): the path to the firmware to flash

    Returns:
      The number of bytes flashed.
    """
    return jlink.flash_file(firmware, 0)


def flash_k21(jlink, firmware):
    """Flashes the given firmware onto K21.

    Args:
      jlink (JLink): the connected ``JLink`` instance
      firmware (str): the path to the firmware to flash

    Returns:
      The number of bytes flashed.
    """
    jlink.power_on()
    jlink.set_reset_strategy(pylink.enums.JLinkResetStrategyCortexM3.RESETPIN)
    jlink.reset()

    if not pylink.unlock_kinetis(jlink):
        jlink.power_off()
        return -1

    res = jlink.flash_file(firmware, 0)
    jlink.reset()
    jlink.power_off()

    return res



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = '2.0.1'
__title__ = 'pylink'
__author__ = 'Square Embedded Software Team'
__author_email__ = 'esw-team@squareup.com'
__copyright__ = 'Copyright 2017 Square, Inc.'
__license__ = 'Apache 2.0'
__url__ = 'http://www.github.com/Square/pylink'
__description__ = 'Python interface for SEGGER J-Link.'
__long_description__ = '''This module provides a Python implementation of the
J-Link SDK by leveraging the SDK's DLL.
'''

from .enums import *
from .errors import *
from .jlink import *
from .library import *
from .structs import *
from .unlockers import *



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import behave


@behave.given('has the license')
@behave.when('I add the license')
def step_add_license(context):
    """Adds a license to the J-Link.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    assert jlink.add_license(str(context.text))


@behave.when('I erase the licenses')
def step_erase_licenses(context):
    """Erases the J-Link's licenses.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    assert jlink.erase_licenses()


@behave.then('my J-Link should have the license')
def step_has_license(context):
    """Asserts the J-Link has the given licenese.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    expected = context.text.strip()
    actual = jlink.custom_licenses.strip()
    assert actual.startswith(expected)
    assert jlink.erase_licenses()


@behave.then('my J-Link should have no licenses')
def step_no_licenses(context):
    """Asserts the J-Link has no licenses.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert len(context.jlink.custom_licenses) == 0



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import behave

MEMORY_ADDR = 0x500


@behave.when('I write the {size}-bit integer {value} to memory')
def step_write_memory(context, size, value):
    """Writes a value to the memory of the device.

    Args:
      context (Context): the ``Context`` instance
      size (str): the number of bits to use for the value
      value (str): the value to write

    Returns:
      ``None``
    """
    jlink = context.jlink
    methods = {
        '8': jlink.memory_write8,
        '16': jlink.memory_write16,
        '32': jlink.memory_write32,
        '64': jlink.memory_write64
    }

    method = methods.get(size, None)
    assert method is not None

    data = [int(value)]
    num_bytes = method(MEMORY_ADDR, data)
    expected_num_bytes = int(size) / 8
    assert num_bytes == expected_num_bytes


@behave.then('I should read the {size}-bit integer {value} from memory')
def step_read_memory(context, size, value):
    """Reads a value from the memory of the device.

    Args:
      context (Context): the ``Context`` instance
      size (str): the number of bits to use for the value
      value (str): the value to read

    Returns:
      ``None``
    """
    jlink = context.jlink
    methods = {
        '8': jlink.memory_read8,
        '16': jlink.memory_read16,
        '32': jlink.memory_read32,
        '64': jlink.memory_read64
    }

    method = methods.get(size, None)
    assert method is not None

    expected = int(value)
    actual = method(MEMORY_ADDR, 1)[0]
    assert expected == actual


@behave.then('I should read the integer {value} from code memory')
def step_read_code_memory(context, value):
    """Reads and checks a value from code memory.

    Args:
      context (Context): the ``Context`` instance
      value (str): the value that should be read

    Returns:
      ``None``
    """
    actual = context.jlink.code_memory_read(MEMORY_ADDR, 1)[0]
    expected = int(value)
    assert actual == expected



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import behave

import time


@behave.given('I enable soft breakpoints')
def step_enable_soft_breakpoints(context):
    """Enables software breakpoints.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    context.jlink.enable_soft_breakpoints()


@behave.when('I set a breakpoint at address {addr}')
def step_set_breakpoint(context, addr):
    """Sets a breakpoint at the given address.

    Args:
      context (Context): the ``Context`` instance
      addr (str): the address to set the breakpoint at

    Returns:
      ``None``

    Raises:
      TypeError: if the given address cannot be converted to an int.
    """
    addr = int(addr, 0)
    context.handle = context.jlink.breakpoint_set(addr, thumb=True)

    assert context.jlink.breakpoint_find(addr) == context.handle

    bp = context.jlink.breakpoint_info(context.handle)
    assert bp.Addr == addr


@behave.then('I should have {num} breakpoint')
@behave.then('I should have {num} breakpoints')
def step_num_breakpoints(context, num):
    """Asserts that there are the specified number of breakpoints.

    Args:
      context (Context): the ``Context`` instance
      num (str): number of breakpoints that should exist

    Returns:
      ``None``

    Raises:
      TypeError: if the given number cannot be converted to an int.
    """
    assert int(num) == context.jlink.num_active_breakpoints()


@behave.then('I should be breakpointed at address {addr}')
def step_should_be_breakpointed(context, addr):
    """Asserts that we are breakpointed at the given address.

    Args:
      context (Context): the ``Context`` instance
      addr (str): the address we should be breakpointed at

    Returns:
      ``None``

    Raises:
      TypeError: if the given address cannot be converted to an int.
    """
    jlink = context.jlink
    assert jlink.halted()

    cpu_halt_reasons = jlink.cpu_halt_reasons()
    assert len(cpu_halt_reasons) == 1

    halt_reason = cpu_halt_reasons[0]
    assert halt_reason.code_breakpoint()

    pc = jlink.register_read(15)
    assert pc == int(addr, 0)


@behave.then('I can clear the breakpoints')
def step_clear_breakpoints(context):
    """Asserts clearing the breakpoints.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.breakpoint_clear_all()


@behave.when('I set a watchpoint at address {addr} for data {data}')
def step_set_watchpoint(context, addr, data):
    """Sets a watchpoint at the given address for the given data value.

    Args:
      context (Context): the ``Context`` instance
      addr (str): the address the watchpoint should be set at
      data (str): the data value to trigger the watchpoint on

    Returns:
      ``None``

    Raises:
      TypeError: if the given address or data cannot be converted to an int.
    """
    addr = int(addr, 0)
    data = int(data, 0)
    context.handle = context.jlink.watchpoint_set(addr, data=data)

    wp = context.jlink.watchpoint_info(context.handle)
    assert wp.Addr == addr
    assert wp.Handle == context.handle
    assert wp.Data == data


@behave.then('I should have {num} watchpoint')
@behave.then('I should have {num} watchpoints')
def step_num_watchpoints(context, num):
    """Asserts that there are the specified number of watchpoints.

    Args:
      context (Context): the ``Context`` instance
      num (str): number of watchpoints that should exist

    Returns:
      ``None``

    Raises:
      TypeError: if the given number cannot be converted to an int.
    """
    assert int(num) == context.jlink.num_active_watchpoints()


@behave.then('I should hit the watchpoint at address {addr}')
def step_should_hit_watchpoint(context, addr):
    """Checks that the watchpoint is hit at the given address.

    Args:
      context (Context): the ``Context`` instance
      addr (str): the address the watchpoint is expected to trigger on

    Returns:
      ``None``

    Raises:
      TypeError: if the given address cannot be converted to an int.
    """
    jlink = context.jlink

    while not jlink.halted():
        time.sleep(1)

    cpu_halt_reasons = jlink.cpu_halt_reasons()
    assert len(cpu_halt_reasons) == 1

    halt_reason = cpu_halt_reasons[0]
    assert halt_reason.data_breakpoint()

    index = halt_reason.Index
    wp = jlink.watchpoint_info(index=index)
    addr = int(addr, 0)
    assert addr == wp.Addr


@behave.then('I can clear the watchpoints')
def step_clear_watchpoints(context):
    """Asserts clearing the watchpoints.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.watchpoint_clear_all()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.enums as enums

import unittest


class TestEnums(unittest.TestCase):
    """Unit test for the `enums` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        pass

    def test_jlink_global_errors(self):
        """Tests the global errors.

        The J-Link DLL defines a set of global error codes, which start at -256
        and run until -274.

        Values from -1 to -255 are reserved for function specific error codes.

        This test checks that an error message is defined for those from -256
        to -274, and not for -1 to -255.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        for i in range(-255, -1):
            with self.assertRaises(ValueError):
                enums.JLinkGlobalErrors.to_string(i)

        for i in range(-274, -255):
            error_message = enums.JLinkGlobalErrors.to_string(i)
            self.assertTrue(isinstance(error_message, str))

    def test_jlink_global_errors_unspecified(self):
        """Tests the unspecified error case.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        error_message = enums.JLinkGlobalErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)

    def test_jlink_write_errors(self):
        """Tests write specific errors.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        error_message = enums.JLinkWriteErrors.to_string(-5)
        self.assertTrue(isinstance(error_message, str))

        error_message = enums.JLinkWriteErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)

    def test_jlink_read_errors(self):
        """Tests read specific errors.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        error_message = enums.JLinkReadErrors.to_string(-5)
        self.assertTrue(isinstance(error_message, str))

        error_message = enums.JLinkReadErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)

    def test_jlink_data_event_errors(self):
        """Tests the data event errors.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        error_codes = [
            0x80000000, 0x80000001, 0x80000002, 0x80000004, 0x80000020,
            0x80000040, 0x80000080
        ]

        for error_code in error_codes:
            error_message = enums.JLinkDataErrors.to_string(error_code)
            self.assertTrue(isinstance(error_message, str))

        error_message = enums.JLinkDataErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)

    def test_jlink_flash_errors(self):
        """Tests flash specific errors.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        for i in range(-4, -1):
            error_message = enums.JLinkFlashErrors.to_string(i)
            self.assertTrue(isinstance(error_message, str))

        error_message = enums.JLinkFlashErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)

    def test_jlink_erase_errors(self):
        """Tests erase specific errors.

        Args:
          self (TestEnums): the ``TestEnums`` instance

        Returns:
          ``None``
        """
        for i in range(-5, -4):
            error_message = enums.JLinkEraseErrors.to_string(i)
            self.assertTrue(isinstance(error_message, str))

        error_message = enums.JLinkEraseErrors.to_string(-1)
        self.assertEqual('Unspecified error.', error_message)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.unlockers as unlock

import mock

import unittest


class TestUnlock(unittest.TestCase):
    """Tests the `unlockers` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestUnlock): the `TestUnlock` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestUnlock): the `TestUnlock` instance

        Returns:
          `None`
        """
        pass

    def test_unlock_unsupported(self):
        """Tests calling `unlock()` for an unsupported MCU.

        Args:
          self (TestUnlock): the `TestUnlock` instance

        Returns:
          `None`
        """
        jlink = mock.Mock()
        with self.assertRaises(NotImplementedError):
            unlock.unlock(jlink, 'kinetisf')

        with self.assertRaises(NotImplementedError):
            unlock.unlock(jlink, 'dsafdsafdas')

    @mock.patch('pylink.unlockers.unlock_kinetis')
    def test_unlock_supported(self, mock_unlock):
        """Tests calling `unlock()` with a supported MCU.

        Args:
          self (TestUnlock): the `TestUnlock` instance
          mock_unlock (function): the mocked unlock function

        Returns:
          `None`
        """
        jlink = mock.Mock()
        supported = ['Kinetis', 'kinetis', 'NXP']
        mock_unlock.return_value = True
        for mcu in supported:
            self.assertTrue(unlock.unlock(jlink, mcu))


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import utility

import pylink

import behave

import os
import multiprocessing
import sys
import time


@behave.given('my J-link is connected')
def step_connected(context):
    """Asserts that the J-Link is connected.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.connected()


@behave.given('device {device}')
def step_connect_device(context, device):
    """Tries to connect to the given.

    Args:
      context (Context): the ``Context`` instance
      device (str): name of the devie to connect to

    Returns:
      ``None``
    """
    try:
        jlink = context.jlink
        jlink.connect(str(device))
        jlink.set_reset_strategy(pylink.JLinkResetStrategyCortexM3.NORMAL)
    except pylink.JLinkException as e:
        if e.code == pylink.JLinkGlobalErrors.VCC_FAILURE:
            context.scenario.skip(reason='Target is not powered.')
        elif e.code == pylink.JLinkGlobalErrors.NO_CPU_FOUND:
            context.scenario.skip(reason='Target core not found.')


@behave.given('target interface {interface}')
def step_target_interface(context, interface):
    """Sets the target interface for the JLink.

    Args:
      context (Context): the ``Context`` instance
      interface (str): the interface to use for the J-Link

    Returns:
      ``None``
    """
    if interface.lower() == 'jtag':
        context.jlink.set_tif(pylink.enums.JLinkInterfaces.JTAG)
    elif interface.lower() == 'swd':
        context.jlink.set_tif(pylink.enums.JLinkInterfaces.SWD)
    else:
        assert False, 'Unknown interface: %s' % interface


@behave.given('I close the J-Link')
def step_close(context):
    """Closes the J-Link.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    context.jlink.close()


@behave.when('I reset my device')
def step_device_reset(context):
    """Resets the target connected to the J-Link.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    jlink.power_on()
    jlink.coresight_configure()
    jlink.reset(halt=False)
    time.sleep(1)


@behave.when('I flash the firmware {firmware}')
def step_flash_firmware_when(context, firmware):
    """Tries to flash the firmware.

    Args:
      context (Context): the ``Context`` instance
      firmware (str): the name of the firmware to flash

    Returns:
      ``None``
    """
    return step_flash_firmware_retries(context, firmware, 0)


@behave.when('I read the firmware {firmware} into a bytestream')
def step_read_firmware(context, firmware):
    """Reads the firmware file into a bytestream.

    Args:
      context (Context): the ``Context`` instance
      firmware (str): the name of the firmware

    Returns:
      ``None``
    """
    firmware = utility.firmware_path(str(firmware))
    data = []
    with open(firmware, 'rb') as f:
        byte = f.read(1)
        while byte:
            data.append(ord(byte))
            byte = f.read(1)
    context.data = data


@behave.when('I halt the device')
def step_halt(context):
    """Halts the J-Link target.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    context.jlink.halt()


@behave.when('I create a new process with my J-Link')
def step_create_process(context):
    """Creates a new process to access the J-Link.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    context.process = multiprocessing.Process


@behave.then('the device should be halted')
def step_is_halted(context):
    """Assets that the device is halted / no longer running.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.halted()


@behave.then('the emulator should be connected')
def step_not_connected(context):
    """Checks that the JLink is not open.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.opened()
    assert context.jlink.connected()


@behave.then('the emulator should not be connected')
def step_not_connected(context):
    """Checks that the JLink is not open.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    assert not context.jlink.opened()
    assert not context.jlink.connected()


@behave.then('I can flash the firmware {firmware} with 1 retry')
def step_flash_firmware(context, firmware):
    """Tries to flash the firmware.

    Args:
      context (Context): the ``Context`` instance
      firmware (str): the name of the firmware to flash

    Returns:
      ``None``
    """
    return step_flash_firmware_retries(context, firmware, 1)


@behave.then('I can flash the firmware with {retries} retries')
def step_flash_firmware_bytestream_retries(context, retries):
    """Tries to flash the firmware from a bytestream.

    Args:
      context (Context): the ``Context`` instance
      retries (int): the number of retries when flashing

    Returns:
      ``None``
    """
    jlink = context.jlink
    retries = int(retries)
    data = context.data

    while retries >= 0:
        try:
            res = jlink.flash(context.data, 0)
            if res >= 0:
                break
        except pylink.JLinkException as e:
            retries = retries - 1

    assert retries >= 0

    written = list(map(int, jlink.memory_read8(0, len(data))))
    assert written == data


@behave.then('I can flash the firmware {firmware} with {retries} retries')
def step_flash_firmware_retries(context, firmware, retries):
    """Tries to flash the firmware with the given number of retries.

    Args:
      context (Context): the ``Context`` instance
      firmware (str): the name of the firmware to flash
      retries (int): the number of retries to do

    Returns:
      ``None``
    """
    jlink = context.jlink
    retries = int(retries)
    firmware = utility.firmware_path(str(firmware))
    assert firmware is not None

    while True:
        try:
            res = utility.flash_k21(jlink, firmware)
            if res >= 0:
                break
            else:
                retries = retries - 1
        except pylink.JLinkException as e:
            sys.stderr.write('%s%s' % (str(e), os.linesep))
            retries = retries - 1

        if retries < 0:
            break

    assert retries >= 0


@behave.then('a new J-Link should not be connected')
def step_new_jlink_not_connected(context):
    """Assserts that a new J-Link instance will not be connected.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink_a, jlink_b = context.jlink, pylink.JLink()

    assert not jlink_b.opened()
    assert jlink_a.opened() != jlink_b.opened()

    assert not jlink_b.connected()
    assert jlink_a.connected() != jlink_b.connected()

    assert jlink_a.num_connected_emulators() == jlink_b.num_connected_emulators()

    jlink_b.close()

    assert jlink_a.opened()
    assert jlink_a.connected()


@behave.then('that process\'s J-Link is also connected')
def step_process_jlink_connected(context):
    """Asserts that the J-Link is connected in the second process.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    def f(jlink, queue):
        queue.put(jlink.opened())
        queue.put(jlink.connected())

    queue = multiprocessing.Queue()
    process = context.process(target=f, args=(context.jlink, queue))

    process.start()
    process.join()

    assert all(queue.get() for _ in range(2))


@behave.then('I should not be able to open a new connection to it')
def step_connection_open_jlink(context):
    """Asserts that we cannot open a new connection to an opened J-Link.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    serial_no = context.jlink.serial_number
    new_jlink = pylink.JLink()

    try:
        new_jlink.open(serial_no=serial_no)
    except pylink.JLinkException as e:
        pass
    else:
        assert False



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.structs as structs

import ctypes
import unittest


class TestStructs(unittest.TestCase):
    """Unit test for the ``structs`` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        pass

    def test_jlink_connect_info(self):
        """Tests the ``JLinkConnectInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        info = structs.JLinkConnectInfo()
        info.SerialNumber = 3042517899
        info.acProduct = str.encode('J-Link Plus')
        info.Connection = 1

        info_string = 'J-Link Plus <Serial No. %s, Conn. USB>' % (info.SerialNumber)
        self.assertEqual(info_string, str(info))
        self.assertEqual('JLinkConnectInfo(%s)' % info_string, repr(info))

        info.Connection = 0
        info_string = 'J-Link Plus <Serial No. %s, Conn. IP>' % (info.SerialNumber)
        self.assertEqual(info_string, str(info))
        self.assertEqual('JLinkConnectInfo(%s)' % info_string, repr(info))

    def test_jlink_flash_area(self):
        """Tests the ``JLinkFlashArea`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        flash_area = structs.JLinkFlashArea()
        flash_area.Addr = int(0xDEADBEEF)
        flash_area.Size = 1337

        flash_area_string = 'Address = 0xdeadbeef, Size = 1337'
        self.assertEqual(flash_area_string, str(flash_area))
        self.assertEqual('JLinkFlashArea(%s)' % flash_area_string, repr(flash_area))

    def test_jlink_ram_area(self):
        """Tests the ``JLinkRAMArea`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        ram_area = structs.JLinkRAMArea()
        ram_area.Addr = int(0xDEADBEEF)
        ram_area.Size = 1337

        ram_area_string = 'Address = 0xdeadbeef, Size = 1337'
        self.assertEqual(ram_area_string, str(ram_area))
        self.assertEqual('JLinkRAMArea(%s)' % ram_area_string, repr(ram_area))

    def test_jlink_device_info(self):
        """Tests the ``JLinkDeviceInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        info = structs.JLinkDeviceInfo()
        self.assertEqual(info.SizeofStruct, ctypes.sizeof(info))

        name = b'A Feast For Crows'
        info.sName = ctypes.cast(name, ctypes.POINTER(ctypes.c_char))
        self.assertEqual(name.decode(), info.name)

        manufacturer = b'G.R.R. Martin'
        info.sManu = ctypes.cast(manufacturer, ctypes.POINTER(ctypes.c_char))
        self.assertEqual(manufacturer.decode(), info.manufacturer)

        info_string = 'A Feast For Crows <Core Id. 0, Manu. G.R.R. Martin>'
        self.assertEqual(info_string, str(info))
        self.assertEqual('JLinkDeviceInfo(%s)' % info_string, repr(info))

    def test_jlink_hardware_status(self):
        """Tests the ``JLinkHardwareStatus`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        voltage = 100
        status = structs.JLinkHardwareStatus()
        status.VTarget = voltage
        self.assertEqual(voltage, status.voltage)
        self.assertEqual('JLinkHardwareStatus(VTarget=100mV)', str(status))
        self.assertEqual('JLinkHardwareStatus(VTarget=100mV)', repr(status))

    def test_jlink_gpio_descriptor(self):
        """Tests the ``JLinkGPIODescriptor`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns
          ``None``
        """
        name = 'Samuel L. Jackson'
        gpio = structs.JLinkGPIODescriptor()
        gpio.acName = name.encode()

        self.assertEqual(name, str(gpio))
        self.assertEqual('JLinkGPIODescriptor(%s)' % name, repr(gpio))

    def test_jlink_memory_zone(self):
        """Tests the ``JLinkMemoryZone`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        name = b'Futurama'
        desc = b'A show about future stuff'
        addr = 0xdeadbeef

        memory_zone = structs.JLinkMemoryZone()
        memory_zone.sName = ctypes.cast(name, ctypes.c_char_p)
        memory_zone.sDesc = ctypes.cast(desc, ctypes.c_char_p)
        memory_zone.VirtAddr = addr

        memory_zone_string = '%s <Desc. %s, VirtAddr. 0x%x>' % (name, desc, addr)
        self.assertEqual(name, memory_zone.name)
        self.assertEqual(memory_zone_string, str(memory_zone))
        self.assertEqual('JLinkMemoryZone(%s)' % memory_zone_string, repr(memory_zone))

    def test_jlink_speed_info(self):
        """Tests the ``JLinkSpeedInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        info = structs.JLinkSpeedInfo()

        self.assertEqual(ctypes.sizeof(info), info.SizeOfStruct)

        info_string = 'JLinkSpeedInfo(Freq=0Hz)'
        self.assertEqual(info_string, str(info))
        self.assertEqual(info_string, repr(info))

    def test_jlink_swo_start_info(self):
        """Tests the ``JLinkSWOStartInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        start_info = structs.JLinkSWOStartInfo()
        self.assertEqual(ctypes.sizeof(start_info), start_info.SizeofStruct)
        self.assertEqual(0, start_info.Interface)

        info_string = 'JLinkSWOStartInfo(Speed=0Hz)'
        self.assertEqual(info_string, repr(start_info))
        self.assertEqual(info_string, str(start_info))

    def test_jlink_swo_speed_info(self):
        """Tests the ``JLinkSWOSpeedInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        speed_info = structs.JLinkSWOSpeedInfo()
        self.assertEqual(ctypes.sizeof(speed_info), speed_info.SizeofStruct)
        self.assertEqual(0, speed_info.Interface)

        speed_string = 'JLinkSWOSpeedInfo(Interface=UART, Freq=0Hz)'
        self.assertEqual(speed_string, repr(speed_info))
        self.assertEqual(speed_string, str(speed_info))

    def test_jlink_moe_info(self):
        """Tests the ``JLinkMOEInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        moe_info = structs.JLinkMOEInfo()

        # DBGRQ
        moe_info.HaltReason = 0
        self.assertTrue(moe_info.dbgrq())
        self.assertFalse(moe_info.code_breakpoint())
        self.assertFalse(moe_info.data_breakpoint())
        self.assertFalse(moe_info.vector_catch())
        self.assertEqual('JLinkMOEInfo(DBGRQ)', repr(moe_info))
        self.assertEqual('DBGRQ', str(moe_info))

        # Code Breakpoint
        moe_info.HaltReason = 1
        self.assertFalse(moe_info.dbgrq())
        self.assertTrue(moe_info.code_breakpoint())
        self.assertFalse(moe_info.data_breakpoint())
        self.assertFalse(moe_info.vector_catch())
        self.assertEqual('JLinkMOEInfo(Code Breakpoint)', repr(moe_info))
        self.assertEqual('Code Breakpoint', str(moe_info))

        # Data Breakpoint
        moe_info.HaltReason = 2
        self.assertFalse(moe_info.dbgrq())
        self.assertFalse(moe_info.code_breakpoint())
        self.assertTrue(moe_info.data_breakpoint())
        self.assertFalse(moe_info.vector_catch())
        self.assertEqual('JLinkMOEInfo(Data Breakpoint)', repr(moe_info))
        self.assertEqual('Data Breakpoint', str(moe_info))

        # Vector Catch
        moe_info.HaltReason = 3
        self.assertFalse(moe_info.dbgrq())
        self.assertFalse(moe_info.code_breakpoint())
        self.assertFalse(moe_info.data_breakpoint())
        self.assertTrue(moe_info.vector_catch())
        self.assertEqual('JLinkMOEInfo(Vector Catch)', repr(moe_info))
        self.assertEqual('Vector Catch', str(moe_info))

    def test_jlink_breakpoint_info(self):
        """Tests the ``JLinkBreakpointInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        bp = structs.JLinkBreakpointInfo()
        self.assertEqual(ctypes.sizeof(bp), bp.SizeOfStruct)
        self.assertFalse(bp.software_breakpoint())
        self.assertFalse(bp.hardware_breakpoint())
        self.assertFalse(bp.pending())

        rep = 'JLinkBreakpointInfo(Handle 0, Address 0)'
        self.assertEqual(rep, str(bp))
        self.assertEqual(rep, repr(bp))

    def test_jlink_data_event(self):
        """Tests the ``JLinkDataEvent`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        event = structs.JLinkDataEvent()
        self.assertEqual(1 << 0, event.Type)
        self.assertEqual(ctypes.sizeof(event), event.SizeOfStruct)

        rep = 'JLinkDataEvent(Type 1, Address 0)'
        self.assertEqual(rep, repr(event))
        self.assertEqual(rep, str(event))

    def test_jlink_watchpoint_info(self):
        """Tests the ``JLinkWatchpointInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        wp = structs.JLinkWatchpointInfo()
        self.assertEqual(ctypes.sizeof(wp), wp.SizeOfStruct)

        rep = 'JLinkWatchpointInfo(Handle 0, Address 0)'
        self.assertEqual(rep, repr(wp))
        self.assertEqual(rep, str(wp))

    def test_jlink_strace_event_info(self):
        """Tests the ``JLinkStraceEventInfo`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        event_info = structs.JLinkStraceEventInfo()
        self.assertEqual(ctypes.sizeof(event_info), event_info.SizeOfStruct)

        rep = 'JLinkStraceEventInfo(Type=0, Op=0)'
        self.assertEqual(rep, str(event_info))
        self.assertEqual(rep, repr(event_info))

    def test_jlink_trace_data(self):
        """Tests the ``JLinkTraceData`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        data = structs.JLinkTraceData()
        rep = 'JLinkTraceData(0)'
        self.assertEqual(rep, str(data))
        self.assertEqual(rep, repr(data))

        pipe_methods = [
            data.instruction,
            data.data_instruction,
            data.non_instruction,
            data.wait,
            data.branch,
            data.data_branch,
            data.trigger,
            data.trace_disabled
        ]

        for i in range(len(pipe_methods)):
            data.PipeStat = i
            for (j, method) in enumerate(pipe_methods):
                if i == j:
                    self.assertTrue(method())
                else:
                    self.assertFalse(method())

        return None

    def test_jlink_trace_region(self):
        """Tests the ``JLinkTraceRegion`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        region = structs.JLinkTraceRegion()
        self.assertEqual(ctypes.sizeof(region), region.SizeOfStruct)

        rep = 'JLinkTraceRegion(Index=0)'
        self.assertEqual(rep, str(region))
        self.assertEqual(rep, repr(region))

    def test_jlink_rtt_terminal_start(self):
        """Tests the ``JLinkRTTerminalStart`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        start = structs.JLinkRTTerminalStart()
        rep = 'JLinkRTTerminalStart(ConfigAddress=0x0)'
        self.assertEqual(rep, repr(start))

        start.ConfigBlockAddress = 0xDEADBEEF
        rep = 'JLinkRTTerminalStart(ConfigAddress=0xDEADBEEF)'
        self.assertEqual(rep, str(start))

    def test_jlink_rtt_terminal_buf_desc(self):
        """Tests the ``JLinkRTTerminalBufDesc`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        desc = structs.JLinkRTTerminalBufDesc()
        desc.BufferIndex = 7
        desc.SizeOfBuffer = 1337
        desc.Flags = 0
        desc.acName = str.encode('The Winds of Winter')

        desc.Direction = 0
        self.assertTrue(desc.up)
        self.assertFalse(desc.down)

        desc.Direction = 1
        self.assertTrue(desc.down)
        self.assertFalse(desc.up)

        self.assertEqual('The Winds of Winter', desc.name)
        self.assertEqual('JLinkRTTerminalBufDesc(Index=7, Name=The Winds of Winter)', repr(desc))
        self.assertEqual('The Winds of Winter <Index=7, Direction=down, Size=1337>', str(desc))

        desc.Direction = 0
        self.assertEqual('The Winds of Winter <Index=7, Direction=up, Size=1337>', str(desc))

    def test_jlink_rtt_terminal_stat(self):
        """Tests the ``JLinkRTTerminalStatus`` structure.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        stat = structs.JLinkRTTerminalStatus()
        stat.NumUpBuffers = 3
        stat.NumDownBuffers = 3
        stat.IsRunning = 1

        self.assertEqual('JLinkRTTerminalStatus(NumUpBuffers=3, NumDownBuffers=3)', repr(stat))
        self.assertEqual('Status <NumUpBuffers=3, NumDownBuffers=3, Running=1>', str(stat))

    def test_jlink_power_trace_setup(self):
        """Validates the ``JLinkPowerTraceSetup`` serializes correctly.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        setup = structs.JLinkPowerTraceSetup()
        setup.ChannelMask = 0x3
        setup.SampleFreq = 1000

        self.assertEqual(20, setup.SizeOfStruct)
        self.assertEqual('JLinkPowerTraceSetup(Channel Mask=0b11, Freq=1000Hz)', repr(setup))

    def test_jlink_power_trace_item(self):
        """Validates the ``JLinkPowerTraceItem`` serializes correctly.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        item = structs.JLinkPowerTraceItem()
        item.RefValue = 0xDEADBEEF
        item.Value = 13

        self.assertEqual('JLinkPowerTraceItem(Value=13, Reference=3735928559)', repr(item))

    def test_jlink_power_trace_caps(self):
        """Validates that the ``JLinkPowerTraceCaps`` serializes correctly.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        caps = structs.JLinkPowerTraceCaps()
        caps.ChannelMask = 0x3

        self.assertEqual(8, caps.SizeOfStruct)
        self.assertEqual('JLinkPowerTraceCaps(Channel Mask=0b11)', repr(caps))

    def test_jlink_power_trace_channel_caps(self):
        """Validates the ``JLinkPowerTraceChannelCaps`` instance.

        Args:
          self (TestStructs): the ``TestStructs`` instance

        Returns:
          ``None``
        """
        channel_caps = structs.JLinkPowerTraceChannelCaps()
        channel_caps.BaseSampleFreq = 1000
        channel_caps.MinDiv = 4

        self.assertEqual(12, channel_caps.SizeOfStruct)
        self.assertEqual(250, channel_caps.max_sample_freq)
        self.assertEqual('JLinkPowerTraceChannelCaps(SampleFreq=1000Hz, MinDiv=4)', repr(channel_caps))


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import threads

import functools


def async_decorator(func):
    """Asynchronous function decorator.  Interprets the function as being
    asynchronous, so returns a function that will handle calling the
    Function asynchronously.

    Args:
      func (function): function to be called asynchronously

    Returns:
      The wrapped function.

    Raises:
      AttributeError: if ``func`` is not callable
    """

    @functools.wraps(func)
    def async_wrapper(*args, **kwargs):
        """Wraps up the call to ``func``, so that it is called from a separate
        thread.

        The callback, if given, will be called with two parameters,
        ``exception`` and ``result`` as ``callback(exception, result)``.  If
        the thread ran to completion without error, ``exception`` will be
        ``None``, otherwise ``exception`` will be the generated exception that
        stopped the thread.  Result is the result of the exected function.

        Args:
          callback (function): the callback to ultimately be called
          args: list of arguments to pass to ``func``
          kwargs: key-word arguments dictionary to pass to ``func``

        Returns:
          A thread if the call is asynchronous, otherwise the the return value
          of the wrapped function.

        Raises:
          TypeError: if ``callback`` is not callable or is missing
        """
        if 'callback' not in kwargs or not kwargs['callback']:
            return func(*args, **kwargs)

        callback = kwargs.pop('callback')

        if not callable(callback):
            raise TypeError('Expected \'callback\' is not callable.')

        def thread_func(*args, **kwargs):
            """Thread function on which the given ``func`` and ``callback``
            are executed.

            Args:
              args: list of arguments to pass to ``func``
              kwargs: key-word arguments dictionary to pass to ``func``

            Returns:
              Return value of the wrapped function.
            """
            exception, res = None, None
            try:
                res = func(*args, **kwargs)
            except Exception as e:
                exception = e
            return callback(exception, res)

        thread = threads.ThreadReturn(target=thread_func,
                                      args=args,
                                      kwargs=kwargs)
        thread.daemon = True
        thread.start()
        return thread

    return async_wrapper



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import behave

import string
import time


@behave.when('I enable SWO')
def step_enable_swo(context):
    """Enables Serial Wire Output.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    return step_enable_swo_on_port(context, 0)


@behave.when('I enable SWO on port {port}')
def step_enable_swo_on_port(context, port):
    """Enables Serial Wire Output.

    Args:
      context (Context): the ``Context`` instance
      port (int): the port SWO is enabled on

    Returns:
      ``None``
    """
    jlink = context.jlink

    # Have to halt the CPU before getting its speed.
    jlink.reset()
    jlink.halt()
    assert jlink.halted()

    cpu_speed = jlink.cpu_speed()
    swo_speed = jlink.swo_supported_speeds(cpu_speed, 3)[0]

    # Enable SWO and flush.  This must come before the reset to ensure that no
    # data is present in the SWO buffer when the core restarts.
    jlink.swo_start(swo_speed)
    jlink.swo_flush()

    assert jlink.swo_enabled()

    # Reset the core without halting so that it runs.
    jlink.reset(ms=10, halt=False)
    time.sleep(1)


@behave.then('I should see "{message}"')
def step_should_see(context, message):
    """Asserts that the given message was read.

    Args:
      context (Context): the ``Context`` instance
      message (str): the message at the head of the SWO buffer

    Returns:
      ``None``
    """
    data = context.jlink.swo_read_stimulus(0, 0x800)
    actual = ''.join(map(chr, data))

    print('Actual = %s' % actual)
    print('Expected = %s' % message)
    assert len(actual) == len(message)
    assert actual == message



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.errors as errors

import unittest


class TestErrors(unittest.TestCase):
    """Unit test for the `errors` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestErrors): the `TestErrors` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestErrors): the `TestErrors` instance

        Returns:
          `None`
        """
        pass

    def test_jlink_exception_with_message(self):
        """Tests that a `JLinkException` can be created with a message.

        Args:
          self (TestErrors): the `TestErrors` instance

        Returns:
          `None`
        """
        message = 'message'
        exception = errors.JLinkException(message)
        self.assertTrue(isinstance(exception.message, str))
        self.assertEqual(message, exception.message)
        self.assertEqual(None, getattr(exception, 'code', None))

    def test_jlink_exception_with_code(self):
        """Tests that a `JLinkException` can be created with a numeric code.

        Args:
          self (TestErrors): the `TestErrors` instance

        Returns:
          `None`
        """
        code = -1
        exception = errors.JLinkException(code)
        self.assertTrue(isinstance(exception.message, str))
        self.assertEqual('Unspecified error.', exception.message)
        self.assertEqual(code, getattr(exception, 'code', None))


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .. import util

import ctypes


class Response(object):
    """Response class to hold the response from the send of a SWD request."""
    STATUS_ACK = (1 << 0)
    STATUS_WAIT = (1 << 1)
    STATUS_FAULT = (1 << 2)
    STATUS_INVALID = -1

    def __init__(self, status, data=None):
        """Initializes the response.

        Args:
          self (Response): the ``Response`` instance
          status (int): the status the response exited with
          data (object): the optional data returned from the request

        Returns:
          ``None``
        """
        self.status = status
        self.data = data

    def ack(self):
        """Returns whether the response was ACK'd.

        Args:
          self (Response): the ``Response`` instance

        Returns:
          ``True`` if response was ACK'd, otherwise ``False``.
        """
        return (self.status == self.STATUS_ACK)

    def wait(self):
        """Returns whether the response was a wait.

        Args:
          self (Response): the ``Response`` instance

        Returns:
          ``True`` if response exited with wait, otherwise ``False``.
        """
        return (self.status == self.STATUS_WAIT)

    def fault(self):
        """Returns whether the response exited with fault.

        Args:
          self (Response): the ``Response`` instance

        Returns:
          ``True`` if response exited with a fault, otherwise ``False``.
        """
        return (self.status == self.STATUS_FAULT)

    def invalid(self):
        """Returns whether the response exited with a bad result.

        This occurs when the parity is invalid.

        Args:
          self (Response): the ``Response`` instance

        Returns:
          ``True`` if the parity checked failed, otherwise ``False``.
        """
        return (self.status == self.STATUS_INVALID)


class RequestBits(ctypes.LittleEndianStructure):
    """SWD request bits."""
    _fields_ = [
        ('start',      ctypes.c_uint8, 1),
        ('ap_dp',      ctypes.c_uint8, 1),
        ('read_write', ctypes.c_uint8, 1),
        ('addr2',      ctypes.c_uint8, 1),
        ('addr3',      ctypes.c_uint8, 1),
        ('parity',     ctypes.c_uint8, 1),
        ('stop',       ctypes.c_uint8, 1),
        ('park',       ctypes.c_uint8, 1)
    ]


class Request(ctypes.Union):
    """Definition of a SWD (Serial Wire Debug) Request.

    An SWD Request is composed of 8 bits.

    Attributes:
      start: the start bit is always one
      ap_dp: indicates whether the transaction is DP (``0``) or AP (``1``).
      read_write: indicates if the transaction is a read-access (``1``) or a
          write-access (``0``).
      address:
      parity: the parity bit, the bit is used by the target to verify the
          integrity of the request.  Should be ``1`` if bits ``1-4`` contain an
          odd number of ``1``s, otherwise ``0``.
      stop: the stop bit, should always be zero.
      park: the park bit, should always be one.
      value: the overall value of the request.
    """
    _anonymous_ = ('bit', )
    _fields_ = [
        ('bit',   RequestBits),
        ('value', ctypes.c_uint8),
    ]

    def __init__(self, address, ap, data=None):
        """Initializes the SWD request.

        Calculates the parity and sets the ``APnDP``, ``start``, ``stop``,
        ``park`` and ``address`` bits.

        Args:
          self (Request): the ``Request`` instance
          address (int): the register index (``A[3:2]``)
          ap (bool): ``True`` if this request is to an Access Port Access
              Register, otherwise ``False`` for a Debug Port Access Register
          data (int): data to write, if any (indicates a write request)

        Returns:
          ``None``
        """
        super(Request, self).__init__()

        self.start = 1
        self.ap_dp = 1 if ap else 0
        self.read_write = 0 if data is not None else 1
        self.addr2 = (address >> 0) & 1
        self.addr3 = (address >> 1) & 1

        parity = self.ap_dp ^ self.read_write ^ self.addr2 ^ self.addr3
        self.parity = parity

        self.stop = 0
        self.park = 1

        self.data = data

    def send(self, jlink):
        """Starts the SWD transaction.

        Sends the request and recieves an ACK for the request.

        Args:
          self (Request): the ``Request`` instance
          jlink (JLink): the ``JLink`` instance to use for write/read

        Returns:
          The bit position of the ACK response.
        """
        # Send the request over SWD.
        jlink.swd_write8(0xFF, self.value)

        # Receive the ACK.
        return jlink.swd_write(0x0, 0x0, 3)


class ReadRequest(Request):
    """Definition for a SWD (Serial Wire Debug) Read Request."""

    def __init__(self, address, ap):
        """Initializes the base class.

        Args:
          self (ReadRequest): the ``ReadRequest`` instance
          address (int): the register index
          ap (bool): ``True`` if this request is to an Access Port Access
              Register, otherwise ``False`` for a Debug Port Access Register

        Returns:
          ``None``
        """
        super(ReadRequest, self).__init__(address=address, ap=ap)

    def send(self, jlink):
        """Starts the SWD transaction.

        Steps for a Read Transaction:
          1.  First phase in which the request is sent.
          2.  Second phase in which an ACK is received.  This phase consists of
              three bits. An OK response has the value ``1``.
          3.  Once the ACK is received, the data phase can begin.  Consists of
              ``32`` data bits followed by ``1`` parity bit calclulated based
              on all ``32`` data bits.
          4.  After the data phase, the interface must be clocked for at least
              eight cycles to clock the transaction through the SW-DP; this is
              done by reading an additional eight bits (eight clocks).

        Args:
          self (ReadRequest): the ``ReadRequest`` instance
          jlink (JLink): the ``JLink`` instance to use for write/read

        Returns:
          An ``Response`` instance.
        """
        ack = super(ReadRequest, self).send(jlink)

        # Write the read command, then read the data and status.
        jlink.swd_write32(0x0, 0x0)
        jlink.swd_write8(0xFC, 0x0)
        status = jlink.swd_read8(ack) & 7
        data = jlink.swd_read32(ack + 3)

        if status == Response.STATUS_ACK:
            # Check the parity
            parity = jlink.swd_read8(ack + 35) & 1
            if util.calculate_parity(data) != parity:
                return Response(-1, data)

        return Response(status, data)


class WriteRequest(Request):
    """Definition for a SWD (Serial Wire Debug) Write Request."""

    def __init__(self, address, ap, data):
        """Initializes the base class.

        Args:
          self (WriteRequest): the ``WriteRequest`` instance
          address (int): the register index
          ap (bool): ``True`` if this request is to an Access Port Access
              Register, otherwise ``False`` for a Debug Port Access Register

        Returns:
          ``None``
        """
        super(WriteRequest, self).__init__(address=address, ap=ap, data=data)

    def send(self, jlink):
        """Starts the SWD transaction.

        Steps for a Write Transaction:
          1.  First phase in which the request is sent.
          2.  Second phase in which an ACK is received.  This phase consists of
              three bits. An OK response has the value ``1``.
          3.  Everytime the SWD IO may change directions, a turnaround phase is
              inserted.  For reads, this happens after the data phase, while
              for writes this happens after between the acknowledge and data
              phase, so we have to do the turnaround before writing data.  This
              phase consists of two bits.
          4.  Write the data and parity bits.

        Args:
          self (WriteRequest): the ``WriteRequest`` instance
          jlink (JLink): the ``JLink`` instance to use for write/read

        Returns:
          An ``Response`` instance.
        """
        ack = super(WriteRequest, self).send(jlink)

        # Turnaround phase for write.
        jlink.swd_write(0x0, 0x0, 2)

        # Write the data and the parity bits.
        jlink.swd_write32(0xFFFFFFFF, self.data)
        jlink.swd_write8(0xFF, util.calculate_parity(self.data))
        return Response(jlink.swd_read8(ack) & 7)



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import behave


@behave.when('I write {value} to ICE register {register}')
def step_write_to_ice_register(context, value, register):
    """Writes a value to a single ICE register.

    Args:
      context (Context): the ``Context`` instance
      value (str): the value to write to the register
      register (str): the register to write to

    Returns:
      ``None``
    """
    jlink = context.jlink
    register = int(register, 0)
    value = int(value, 0)
    jlink.ice_register_write(register, value)


@behave.when('I write {value} to register {register}')
def step_write_to_register(context, value, register):
    """Writes a value to a single register.

    Args:
      context (Context): the ``Context`` instance
      value (str): the value to write to the register
      register (str): the register to write to

    Returns:
      ``None``
    """
    jlink = context.jlink
    jlink.register_write(int(register), int(value))


@behave.when('I write to the registers')
def step_write_to_registers(context):
    """Writes multiple values to multiple registers.

    The values and registers are loaded from the context's table.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    registers, values = [], []
    for row in context.table:
        registers.append(int(row['register']))
        values.append(int(row['value']))
    jlink.register_write_multiple(registers, values)


@behave.then('ICE register {register} should have the value {value}')
def step_ice_register_has_value(context, register, value):
    """Checks that an ICE register has a particular value.

    Args:
      context (Context): the ``Context`` instance
      register (str): the ICE register to read from
      value (str): the value the ICE register should have

    Returns:
      ``None``
    """
    jlink = context.jlink
    register = int(register, 0)
    expected = int(value, 0)
    assert jlink.ice_register_read(register) == expected


@behave.then('register {register} should have the value {value}')
def step_register_has_value(context, register, value):
    """Reads a single value from a single register and asserts equality.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    actual = jlink.register_read(int(register))
    assert actual == int(value)


@behave.then('I should read from the registers')
def step_registers_have_values(context):
    """Reads multiple values from multiple registers and asserts equality.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    registers, expected_values = [], []
    for row in context.table:
        registers.append(int(row['register']))
        expected_values.append(int(row['value']))
    assert expected_values == jlink.register_read_multiple(registers)



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes


class JLinkGlobalErrors(object):
    """Enumeration for the error codes which any J-Link SDK DLL API-function
    can have as a return value."""
    UNSPECIFIED_ERROR = -1
    EMU_NO_CONNECTION = -256
    EMU_COMM_ERROR = -257
    DLL_NOT_OPEN = -258
    VCC_FAILURE = -259
    INVALID_HANDLE = -260
    NO_CPU_FOUND = -261
    EMU_FEATURE_UNSUPPORTED = -262
    EMU_NO_MEMORY = -263
    TIF_STATUS_ERROR = -264
    FLASH_PROG_COMPARE_FAILED = -265
    FLASH_PROG_PROGRAM_FAILED = -266
    FLASH_PROG_VERIFY_FAILED = -267
    OPEN_FILE_FAILED = -268
    UNKNOWN_FILE_FORMAT = -269
    WRITE_TARGET_MEMORY_FAILED = -270
    DEVICE_FEATURE_NOT_SUPPORTED = -271
    WRONG_USER_CONFIG = -272
    NO_TARGET_DEVICE_SELECTED = -273
    CPU_IN_LOW_POWER_MODE = -274

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given ``error_code``.

        Args:
          cls (JlinkGlobalErrors): the ``JLinkGlobalErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.EMU_NO_CONNECTION:
            return 'No connection to emulator.'
        elif error_code == cls.EMU_COMM_ERROR:
            return 'Emulator connection error.'
        elif error_code == cls.DLL_NOT_OPEN:
            return 'DLL has not been opened.  Did you call \'.connect()\'?'
        elif error_code == cls.VCC_FAILURE:
            return 'Target system has no power.'
        elif error_code == cls.INVALID_HANDLE:
            return 'Given file / memory handle is invalid.'
        elif error_code == cls.NO_CPU_FOUND:
            return 'Could not find supported CPU.'
        elif error_code == cls.EMU_FEATURE_UNSUPPORTED:
            return 'Emulator does not support the selected feature.'
        elif error_code == cls.EMU_NO_MEMORY:
            return 'Emulator out of memory.'
        elif error_code == cls.TIF_STATUS_ERROR:
            return 'Target interface error.'
        elif error_code == cls.FLASH_PROG_COMPARE_FAILED:
            return 'Programmed data differs from source data.'
        elif error_code == cls.FLASH_PROG_PROGRAM_FAILED:
            return 'Programming error occured.'
        elif error_code == cls.FLASH_PROG_VERIFY_FAILED:
            return 'Error while verifying programmed data.'
        elif error_code == cls.OPEN_FILE_FAILED:
            return 'Specified file could not be opened.'
        elif error_code == cls.UNKNOWN_FILE_FORMAT:
            return 'File format of selected file is not supported.'
        elif error_code == cls.WRITE_TARGET_MEMORY_FAILED:
            return 'Could not write target memory.'
        elif error_code == cls.DEVICE_FEATURE_NOT_SUPPORTED:
            return 'Feature not supported by connected device.'
        elif error_code == cls.WRONG_USER_CONFIG:
            return 'User configured DLL parameters incorrectly.'
        elif error_code == cls.NO_TARGET_DEVICE_SELECTED:
            return 'User did not specify core to connect to.'
        elif error_code == cls.CPU_IN_LOW_POWER_MODE:
            return 'Target CPU is in low power mode.'
        elif error_code == cls.UNSPECIFIED_ERROR:
            return 'Unspecified error.'
        raise ValueError('Invalid error code: %d' % error_code)


class JLinkEraseErrors(JLinkGlobalErrors):
    """Enumeration for the error codes generated during an erase operation."""

    ILLEGAL_COMMAND = -5

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given ``error_code``.

        Args:
          cls (JLinkEraseErrors): the ``JLinkEraseErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.ILLEGAL_COMMAND:
            return 'Failed to erase sector.'
        return super(JLinkEraseErrors, cls).to_string(error_code)


class JLinkFlashErrors(JLinkGlobalErrors):
    """Enumeration for the error codes generated during a flash operation."""

    COMPARE_ERROR = -2
    PROGRAM_ERASE_ERROR = -3
    VERIFICATION_ERROR = -4

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given ``error_code``.

        Args:
          cls (JLinkFlashErrors): the ``JLinkFlashErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.COMPARE_ERROR:
            return 'Error comparing flash content to programming data.'
        elif error_code == cls.PROGRAM_ERASE_ERROR:
            return 'Error during program/erase phase.'
        elif error_code == cls.VERIFICATION_ERROR:
            return 'Error verifying programmed data.'
        return super(JLinkFlashErrors, cls).to_string(error_code)


class JLinkWriteErrors(JLinkGlobalErrors):
    """Enumeration for the error codes generated during a write."""

    ZONE_NOT_FOUND_ERROR = -5

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given ``error_code``.

        Args:
          cls (JLinkWriteErrors): the ``JLinkWriteErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.ZONE_NOT_FOUND_ERROR:
            return 'Zone not found'
        return super(JLinkWriteErrors, cls).to_string(error_code)


class JLinkReadErrors(JLinkGlobalErrors):
    """Enumeration for the error codes generated during a read."""

    ZONE_NOT_FOUND_ERROR = -5

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given ``error_code``.

        Args:
          cls (JLinkReadErrors): the ``JLinkReadErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.ZONE_NOT_FOUND_ERROR:
            return 'Zone not found'
        return super(JLinkReadErrors, cls).to_string(error_code)


class JLinkDataErrors(JLinkGlobalErrors):
    """Enumeration for the error codes generated when setting a data event."""

    ERROR_UNKNOWN = 0x80000000
    ERROR_NO_MORE_EVENTS = 0x80000001
    ERROR_NO_MORE_ADDR_COMP = 0x80000002
    ERROR_NO_MORE_DATA_COMP = 0x80000004
    ERROR_INVALID_ADDR_MASK = 0x80000020
    ERROR_INVALID_DATA_MASK = 0x80000040
    ERROR_INVALID_ACCESS_MASK = 0x80000080

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given error code.

        Args:
          cls (JLinkDataErrors): the ``JLinkDataErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.ERROR_UNKNOWN:
            return 'Unknown error.'
        elif error_code == cls.ERROR_NO_MORE_EVENTS:
            return 'There are no more available watchpoint units.'
        elif error_code == cls.ERROR_NO_MORE_ADDR_COMP:
            return 'No more address comparisons can be set.'
        elif error_code == cls.ERROR_NO_MORE_DATA_COMP:
            return 'No more data comparisons can be set.'
        elif error_code == cls.ERROR_INVALID_ADDR_MASK:
            return 'Invalid flags passed for the address mask.'
        elif error_code == cls.ERROR_INVALID_DATA_MASK:
            return 'Invalid flags passed for the data mask.'
        elif error_code == cls.ERROR_INVALID_ACCESS_MASK:
            return 'Invalid flags passed for the access mask.'
        return super(JLinkDataErrors, cls).to_string(error_code)


class JLinkRTTErrors(JLinkGlobalErrors):
    """Enumeration for error codes from RTT."""

    RTT_ERROR_CONTROL_BLOCK_NOT_FOUND = -2

    @classmethod
    def to_string(cls, error_code):
        """Returns the string message for the given error code.

        Args:
          cls (JLinkRTTErrors): the ``JLinkRTTErrors`` class
          error_code (int): error code to convert

        Returns:
          An error string corresponding to the error code.

        Raises:
          ValueError: if the error code is invalid.
        """
        if error_code == cls.RTT_ERROR_CONTROL_BLOCK_NOT_FOUND:
            return 'The RTT Control Block has not yet been found (wait?)'
        return super(JLinkRTTErrors, cls).to_string(error_code)


class JLinkHost(object):
    """Enumeration for the different JLink hosts: currently only IP and USB."""
    USB = (1 << 0)
    IP = (1 << 1)
    USB_OR_IP = USB | IP


class JLinkInterfaces(object):
    """Target interfaces for the J-Link."""
    JTAG = 0
    SWD = 1
    FINE = 3
    ICSP = 4
    SPI = 5
    C2 = 6


class JLinkResetStrategyCortexM3(object):
    """Target reset strategies for the J-Link.

    Attributes:
      NORMAL: default reset strategy, does whatever is best to reset.
      CORE: only the core is reset via the ``VECTRESET`` bit.
      RESETPIN: pulls the reset pin low to reset the core and peripherals.
      CONNECT_UNDER_RESET: J-Link connects to target while keeping reset
        active.  This is recommented for STM32 devices.
      HALT_AFTER_BTL: halt the core after the bootloader is executed.
      HALT_BEFORE_BTL: halt the core before the bootloader is executed.
      KINETIS: performs a normal reset, but also disables the watchdog.
      ADI_HALT_AFTER_KERNEL: sets the ``SYSRESETREQ`` bit in the ``AIRCR`` in
        order to reset the device.
      CORE_AND_PERIPHERALS: sets the ``SYSRESETREQ`` bit in the ``AIRCR``, and
        the ``VC_CORERESET`` bit in the ``DEMCR`` to make sure that the CPU is
        halted immediately after reset.
      LPC1200: reset for LPC1200 devices.
      S3FN60D: reset for Samsung S3FN60D devices.

    Note:
      Please see the J-Link SEGGER Documentation, UM8001, for full information
      about the different reset strategies.
    """
    NORMAL = 0
    CORE = 1
    RESETPIN = 2
    CONNECT_UNDER_RESET = 3
    HALT_AFTER_BTL = 4
    HALT_BEFORE_BTL = 5
    KINETIS = 6
    ADI_HALT_AFTER_KERNEL = 7
    CORE_AND_PERIPHERALS = 8
    LPC1200 = 9
    S3FN60D = 10


class JLinkFunctions(object):
    """Collection of function prototype and type builders for the J-Link SDK
    API calls."""
    LOG_PROTOTYPE = ctypes.CFUNCTYPE(None, ctypes.c_char_p)
    UNSECURE_HOOK_PROTOTYPE = ctypes.CFUNCTYPE(ctypes.c_int,
                                               ctypes.c_char_p,
                                               ctypes.c_char_p,
                                               ctypes.c_uint32)
    FLASH_PROGRESS_PROTOTYPE = ctypes.CFUNCTYPE(None,
                                                ctypes.c_char_p,
                                                ctypes.c_char_p,
                                                ctypes.c_int)


class JLinkCore(object):
    """Enumeration for the different CPU core identifiers.

    These are the possible cores for targets the J-Link is connected to.
    Note that these are bitfields."""
    NONE = 0x00000000
    ANY = 0xFFFFFFFF
    CORTEX_M1 = 0x010000FF
    COLDFIRE = 0x02FFFFFF
    CORTEX_M3 = 0x030000FF
    CORTEX_M3_R1P0 = 0x03000010
    CORTEX_M3_R1P1 = 0x03000011
    CORTEX_M3_R2P0 = 0x03000020
    SIM = 0x04FFFFFF
    XSCALE = 0x05FFFFFF
    CORTEX_M0 = 0x060000FF
    CORTEX_M_V8BASEL = 0x060100FF
    ARM7 = 0x07FFFFFF
    ARM7TDMI = 0x070000FF
    ARM7TDMI_R3 = 0x0700003F
    ARM7TDMI_R4 = 0x0700004F
    ARM7TDMI_S = 0x070001FF
    ARM7TDMI_S_R3 = 0x0700013F
    ARM7TDMI_S_R4 = 0x0700014F
    CORTEX_A8 = 0x080000FF
    CORTEX_A7 = 0x080800FF
    CORTEX_A9 = 0x080900FF
    CORTEX_A12 = 0x080A00FF
    CORTEX_A15 = 0x080B00FF
    CORTEX_A17 = 0x080C00FF
    ARM9 = 0x09FFFFFF
    ARM9TDMI_S = 0x090001FF
    ARM920T = 0x092000FF
    ARM922T = 0x092200FF
    ARM926EJ_S = 0x092601FF
    ARM946E_S = 0x094601FF
    ARM966E_S = 0x096601FF
    ARM968E_S = 0x096801FF
    ARM11 = 0x0BFFFFFF
    ARM1136 = 0x0B36FFFF
    ARM1136J = 0x0B3602FF
    ARM1136J_S = 0x0B3603FF
    ARM1136JF = 0x0B3606FF
    ARM1136JF_S = 0x0B3607FF
    ARM1156 = 0x0B56FFFF
    ARM1176 = 0x0B76FFFF
    ARM1176J = 0x0B7602FF
    ARM1176J_S = 0x0B7603FF
    ARM1176JF = 0x0B7606FF
    ARM1176JF_S = 0x0B7607FF
    CORTEX_R4 = 0x0C0000FF
    CORTEX_R5 = 0x0C0100FF
    RX = 0x0DFFFFFF
    RX610 = 0x0D00FFFF
    RX62N = 0x0D01FFFF
    RX62T = 0x0D02FFFF
    RX63N = 0x0D03FFFF
    RX630 = 0x0D04FFFF
    RX63T = 0x0D05FFFF
    RX621 = 0x0D06FFFF
    RX62G = 0x0D07FFFF
    RX631 = 0x0D08FFFF
    RX210 = 0x0D10FFFF
    RX21A = 0x0D11FFFF
    RX220 = 0x0D12FFFF
    RX230 = 0x0D13FFFF
    RX231 = 0x0D14FFFF
    RX23T = 0x0D15FFFF
    RX111 = 0x0D20FFFF
    RX110 = 0x0D21FFFF
    RX113 = 0x0D22FFFF
    RX64M = 0x0D30FFFF
    RX71M = 0x0D31FFFF
    CORTEX_M4 = 0x0E0000FF
    CORTEX_M7 = 0x0E0100FF
    CORTEX_M_V8MAINL = 0x0E0200FF
    CORTEX_A5 = 0x0F0000FF
    POWER_PC = 0x10FFFFFF
    POWER_PC_N1 = 0x10FF00FF
    POWER_PC_N2 = 0x10FF01FF
    MIPS = 0x11FFFFFF
    MIPS_M4K = 0x1100FFFF
    MIPS_MICROAPTIV = 0x1101FFFF
    EFM8_UNSPEC = 0x12FFFFFF
    CIP51 = 0x1200FFFF


class JLinkDeviceFamily(object):
    """Enumeration for the difference device families.

    These are the possible device families for targets that the J-Link is
    connected to."""
    AUTO = 0
    CORTEX_M1 = 1
    COLDFIRE = 2
    CORTEX_M3 = 3
    SIMULATOR = 4
    XSCALE = 5
    CORTEX_M0 = 6
    ARM7 = 7
    CORTEX_A8 = 8
    CORTEX_A9 = 8
    ARM9 = 9
    ARM10 = 10
    ARM11 = 11
    CORTEX_R4 = 12
    RX = 13
    CORTEX_M4 = 14
    CORTEX_A5 = 15
    POWERPC = 16
    MIPS = 17
    EFM8 = 18
    ANY = 255


class JLinkFlags(object):
    """Enumeration for the different flags that are passed to the J-Link C SDK
    API methods."""
    GO_OVERSTEP_BP = (1 << 0)

    DLG_BUTTON_YES = (1 << 0)
    DLG_BUTTON_NO = (1 << 1)
    DLG_BUTTON_OK = (1 << 2)
    DLG_BUTTON_CANCEL = (1 << 3)

    HW_PIN_STATUS_LOW = 0
    HW_PIN_STATUS_HIGH = 1
    HW_PIN_STATUS_UNKNOWN = 255


class JLinkSWOInterfaces(object):
    """Serial Wire Output (SWO) interfaces."""
    UART = 0
    MANCHESTER = 1  # DO NOT USE


class JLinkSWOCommands(object):
    """Serial Wire Output (SWO) commands."""
    START = 0
    STOP = 1
    FLUSH = 2
    GET_SPEED_INFO = 3
    GET_NUM_BYTES = 10
    SET_BUFFERSIZE_HOST = 20
    SET_BUFFERSIZE_EMU = 21


class JLinkCPUCapabilities(object):
    """Target CPU Cabilities."""
    READ_MEMORY = (1 << 1)
    WRITE_MEMORY = (1 << 2)
    READ_REGISTERS = (1 << 3)
    WRITE_REGISTERS = (1 << 4)
    GO = (1 << 5)
    STEP = (1 << 6)
    HALT = (1 << 7)
    IS_HALTED = (1 << 8)
    RESET = (1 << 9)
    RUN_STOP = (1 << 10)
    TERMINAL = (1 << 11)
    DCC = (1 << 14)
    HSS = (1 << 15)


class JLinkHaltReasons(object):
    """Halt reasons for the CPU.

    Attributes:
      DBGRQ: CPU has been halted because DBGRQ signal asserted.
      CODE_BREAKPOINT: CPU has been halted because of code breakpoint match.
      DATA_BREAKPOINT: CPU has been halted because of data breakpoint match.
      VECTOR_CATCH: CPU has been halted because of vector catch.
    """
    DBGRQ = 0
    CODE_BREAKPOINT = 1
    DATA_BREAKPOINT = 2
    VECTOR_CATCH = 3


class JLinkVectorCatchCortexM3(object):
    """Vector catch types for the ARM Cortex M3.

    Attributes:
      CORE_RESET: The CPU core reset.
      MEM_ERROR: A memory management error occurred.
      COPROCESSOR_ERROR: Usage fault error accessing the Coprocessor.
      CHECK_ERROR: Usage fault error on enabled check.
      STATE_ERROR: Usage fault state error.
      BUS_ERROR: Normal bus error.
      INT_ERROR: Interrupt or exception service error.
      HARD_ERROR: Hard fault error.
    """
    CORE_RESET = (1 << 0)
    MEM_ERROR = (1 << 4)
    COPROCESSOR_ERROR = (1 << 5)
    CHECK_ERROR = (1 << 6)
    STATE_ERROR = (1 << 7)
    BUS_ERROR = (1 << 8)
    INT_ERROR = (1 << 9)
    HARD_ERROR = (1 << 10)


class JLinkBreakpoint(object):
    """J-Link breakpoint types.

    Attributes:
      SW_RAM: Software breakpont located in RAM.
      SW_FLASH: Software breakpoint located in flash.
      SW: Software breakpoint located in RAM or flash.
      HW: Hardware breakpoint.
      ANY: Allows specifying any time of breakpoint.
      ARM: Breakpoint in ARM mode (only available on ARM 7/9 cores).
      THUMB: Breakpoint in THUMB mode (only available on ARM 7/9 cores).
    """
    SW_RAM = (1 << 4)
    SW_FLASH = (1 << 5)
    SW = (0x000000F0)
    HW = (0xFFFFFF00)
    ANY = (0xFFFFFFF0)
    ARM = (1 << 0)
    THUMB = (2 << 0)


class JLinkBreakpointImplementation(object):
    """J-Link breakpoint implementation types.

    Attributes:
      HARD: Hardware breakpoint using a breakpoint unit.
      SOFT: Software breakpoint using a breakpoint instruction.
      PENDING: Breakpoint has not been set yet.
      FLASH: Breakpoint set in flash.
    """
    HARD = (1 << 0)
    SOFT = (1 << 1)
    PENDING = (1 << 2)
    FLASH = (1 << 4)


class JLinkEventTypes(object):
    """J-Link data event types.

    Attributes:
      BREAKPOINT: breakpoint data event.
    """
    BREAKPOINT = (1 << 0)


class JLinkAccessFlags(object):
    """J-Link access types for data events.

    These access types allow specifying the different types of access events
    that should be monitored.

    Attributes:
      READ: specifies to monitor read accesses.
      WRITE: specifies to monitor write accesses.
      PRIVILEGED: specifies to monitor privileged accesses.
      SIZE_8BIT: specifies to monitor an 8-bit access width.
      SIZE_16BIT: specifies to monitor an 16-bit access width.
      SIZE_32BIT: specifies to monitor an 32-bit access width.
    """
    READ = (0 << 0)
    WRITE = (1 << 0)
    PRIV = (1 << 4)
    SIZE_8BIT = (0 << 1)
    SIZE_16BIT = (1 << 1)
    SIZE_32BIT = (2 << 1)


class JLinkAccessMaskFlags(object):
    """J-Link access mask flags.

    Attributes:
      SIZE: specifies to not care about the access size of the event.
      DIR: specifies to not care about the access direction of the event.
      PRIV: specifies to not care about the access privilege of the event.
    """
    SIZE = (3 << 1)
    DIR = (1 << 0)
    PRIV = (1 << 4)


class JLinkStraceCommand(object):
    """STRACE commmands."""
    TRACE_EVENT_SET = 0
    TRACE_EVENT_CLR = 1
    TRACE_EVENT_CLR_ALL = 2
    SET_BUFFER_SIZE = 3


class JLinkStraceEvent(object):
    """STRACE events."""
    CODE_FETCH = 0
    DATA_ACCESS = 1
    DATA_LOAD = 2
    DATA_STORE = 3


class JLinkStraceOperation(object):
    """STRACE operation specifiers."""
    TRACE_START = 0
    TRACE_STOP = 1
    TRACE_INCLUDE_RANGE = 2
    TRACE_EXCLUDE_RANGE = 3


class JLinkTraceSource(object):
    """Sources for tracing."""
    ETB = 0
    ETM = 1
    MTB = 2


class JLinkTraceCommand(object):
    """J-Link trace commands."""
    START = 0x0
    STOP = 0x1
    FLUSH = 0x2
    GET_NUM_SAMPLES = 0x10
    GET_CONF_CAPACITY = 0x11
    SET_CAPACITY = 0x12
    GET_MIN_CAPACITY = 0x13
    GET_MAX_CAPACITY = 0x14
    SET_FORMAT = 0x20
    GET_FORMAT = 0x21
    GET_NUM_REGIONS = 0x30
    GET_REGION_PROPS = 0x31
    GET_REGION_PROPS_EX = 0x32


class JLinkTraceFormat(object):
    """J-Link trace formats.

    Attributes:
      FORMAT_4BIT: 4-bit data.
      FORMAT_8BIT: 8-bit data.
      FORMAT_16BIT: 16-bit data.
      FORMAT_MULTIPLEXED: multiplexing on ETM / buffer link.
      FORMAT_DEMULTIPLEXED: de-multiplexing on ETM / buffer link.
      FORMAT_DOUBLE_EDGE: clock data on both ETM / buffer link edges.
      FORMAT_ETM7_9: ETM7/ETM9 protocol.
      FORMAT_ETM10: ETM10 protocol.
      FORMAT_1BIT: 1-bit data.
      FORMAT_2BIT: 2-bit data.
    """
    FORMAT_4BIT = 0x1
    FORMAT_8BIT = 0x2
    FORMAT_16BIT = 0x4
    FORMAT_MULTIPLEXED = 0x8
    FORMAT_DEMULTIPLEXED = 0x10
    FORMAT_DOUBLE_EDGE = 0x20
    FORMAT_ETM7_9 = 0x40
    FORMAT_ETM10 = 0x80
    FORMAT_1BIT = 0x100
    FORMAT_2BIT = 0x200


class JLinkROMTable(object):
    """The J-Link ROM tables."""
    NONE = 0x100
    ETM = 0x101
    MTB = 0x102
    TPIU = 0x103
    ITM = 0x104
    DWT = 0x105
    FPB = 0x106
    NVIC = 0x107
    TMC = 0x108
    TF = 0x109
    PTM = 0x10A
    ETB = 0x10B
    DBG = 0x10C
    APBAP = 0x10D
    AHBAP = 0x10E
    SECURE = 0x10F


class JLinkRTTCommand(object):
    """RTT commands."""
    START = 0
    STOP = 1
    GETDESC = 2
    GETNUMBUF = 3
    GETSTAT = 4


class JLinkRTTDirection(object):
    """RTT Direction."""
    UP = 0
    DOWN = 1


class JLinkPowerTraceCommand(object):
    """Power trace commands."""
    SETUP = 0
    START = 1
    FLUSH = 2
    STOP = 3
    GET_CAPS = 4
    GET_CHANNEL_CAPS = 5
    GET_NUM_ITEMS = 6


class JLinkPowerTraceRef(object):
    """Reference values to store on power trace capture.

    Attributes:
        NONE: No reference value is stored.
        BYTES: Number of bytes transferred via SWO is stored since capturing
            started.
        TIME: Number of milliseconds since capturing started.
    """
    NONE = 0
    BYTES = 1
    TIME = 2



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import util

import ctypes
import ctypes.util as ctypes_util
import os
import platform
import re
import sys
import tempfile


class Library(object):
    """Wrapper to provide easy access to loading the J-Link SDK DLL.

    This class provides a convenience for finding and loading the J-Link DLL
    across multiple platforms, and accounting for the inconsistencies between
    Windows and nix-based platforms.

    Attributes:
      _standard_calls_: list of names of the methods for the API calls that
        must be converted to standard calling convention on the Windows
        platform.
      JLINK_SDK_NAME: name of the J-Link DLL on nix-based platforms.
      WINDOWS_JLINK_SDK_NAME: name of the J-Link DLL on Windows platforms.
    """

    _standard_calls_ = [
        'JLINK_Configure',
        'JLINK_DownloadFile',
        'JLINK_GetAvailableLicense',
        'JLINK_GetPCode',
        'JLINK_PrintConfig',
        'JLINK_EraseChip',
        'JLINK_SPI_Transfer',
        'JLINK_GetpFunc',
        'JLINK_GetMemZones',
        'JLINK_ReadMemZonedEx',
        'JLINK_SetHookUnsecureDialog',
        'JLINK_WriteMemZonedEx',
        'JLINK_DIALOG_Configure',
        'JLINK_DIALOG_ConfigureEx',
        'JLINK_EMU_GPIO_GetProps',
        'JLINK_EMU_GPIO_GetState',
        'JLINK_EMU_GPIO_SetState',
        'JLINK_EMU_AddLicense',
        'JLINK_EMU_EraseLicenses',
        'JLINK_EMU_GetLicenses',
        'JLINK_HSS_GetCaps',
        'JLINK_HSS_Start',
        'JLINK_HSS_Stop',
        'JLINK_HSS_Read',
        'JLINK_POWERTRACE_Control',
        'JLINK_POWERTRACE_Read',
        'JLINK_RTTERMINAL_Control',
        'JLINK_RTTERMINAL_Read',
        'JLINK_RTTERMINAL_Write',
        'JLINK_STRACE_Config',
        'JLINK_STRACE_Control',
        'JLINK_STRACE_Read',
        'JLINK_STRACE_Start',
        'JLINK_STRACE_Stop',
        'JLINK_SWD_GetData',
        'JLINK_SWD_GetU8',
        'JLINK_SWD_GetU16',
        'JLINK_SWD_GetU32',
        'JLINK_SWD_StoreGetRaw',
        'JLINK_SWD_StoreRaw',
        'JLINK_SWD_SyncBits',
        'JLINK_SWD_SyncBytes',
        'JLINK_SetFlashProgProgressCallback',
        'JLINKARM_BeginDownload',
        'JLINKARM_EndDownload',
        'JLINKARM_WriteMem'
    ]

    # On Linux and macOS, represents the library file name prefix
    # (the part just before .so or .dylib).
    #
    # This is the value appropriate when calling the find_library_{linux,darwin}()
    # functions below.
    #
    # Note: this "constant" is also used by downstream projects,
    # and should therefore be considered public (frozen) API.
    JLINK_SDK_NAME = 'libjlinkarm'

    # Linux/MacOS: The JLink shared object name, without any prefix like lib,
    # suffix like .so, .dylib or version number.
    #
    # This is the value appropriate when invoking the ctypes API find_libary().
    JLINK_SDK_OBJECT = 'jlinkarm'

    # Windows: these are suitable for both the ctypes find_library() API,
    # and for the directory scanning done in Library.find_library_windows()
    WINDOWS_32_JLINK_SDK_NAME = 'JLinkARM'
    WINDOWS_64_JLINK_SDK_NAME = 'JLink_x64'

    # Linux: Represents the dlinfo(3) associated to the jlinkarm shared library.
    #
    # Poor man's singleton: having this as a class member avoids to create
    # a new JLinkarmDlInfo() for each pylink.Library instance.
    #
    # For e.g., the simple 'pyocd list' command creates three instances
    # of the pylink.Library class, and it's worth not repeating the dlinfo()
    # dance three times.
    _dlinfo = None

    @classmethod
    def get_appropriate_windows_sdk_name(cls):
        """Returns the appropriate JLink SDK library name on Windows depending
        on 32bit or 64bit Python variant.

        SEGGER delivers two variants of their dynamic library on Windows:
          - ``JLinkARM.dll`` for 32-bit platform
          - ``JLink_x64.dll`` for 64-bit platform

        Args:
          cls (Library): the ``Library`` class

        Returns:
          The name of the library depending on the platform this module is run on.

        """
        if sys.maxsize == (2**63 - 1):
            return Library.WINDOWS_64_JLINK_SDK_NAME
        else:
            return Library.WINDOWS_32_JLINK_SDK_NAME

    @classmethod
    def can_load_library(cls, dllpath):
        """Test whether a library is the correct architecture to load.

        Args:
          cls (Library): the ``Library`` class
          dllpath (str): A path to a library.

        Returns:
          ``True`` if the library could be successfully loaded, ``False`` if not.
        """
        try:
            ctypes.CDLL(dllpath)
            return True
        except OSError:
            return False

    @classmethod
    def find_library_windows(cls):
        """Loads the SEGGER DLL from the windows installation directory.

        On Windows, these are found either under:
          - ``C:\\Program Files\\SEGGER\\JLink``
          - ``C:\\Program Files (x86)\\SEGGER\\JLink``.

        Args:
          cls (Library): the ``Library`` class

        Returns:
          The paths to the J-Link library files in the order that they are
          found.
        """
        dll = cls.get_appropriate_windows_sdk_name() + '.dll'
        root = 'C:\\'
        for d in os.listdir(root):
            dir_path = os.path.join(root, d)

            # Have to account for the different Program Files directories.
            if d.startswith('Program Files') and os.path.isdir(dir_path):
                dir_path = os.path.join(dir_path, 'SEGGER')
                if not os.path.isdir(dir_path):
                    continue

                # Find all the versioned J-Link directories.
                ds = filter(lambda x: x.startswith('JLink'), os.listdir(dir_path))
                for jlink_dir in ds:
                    # The DLL always has the same name, so if it is found, just
                    # return it.
                    lib_path = os.path.join(dir_path, jlink_dir, dll)
                    if os.path.isfile(lib_path):
                        yield lib_path

    @classmethod
    def find_library_linux(cls):
        """Loads the SEGGER DLL from the root directory.

        On Linux, the SEGGER tools are installed under the ``/opt/SEGGER``
        directory with versioned directories having the suffix ``_VERSION``.

        Args:
          cls (Library): the ``Library`` class

        Returns:
          The paths to the J-Link library files in the order that they are
          found.
        """
        dll = Library.JLINK_SDK_NAME
        root = os.path.join('/', 'opt', 'SEGGER')

        for (directory_name, subdirs, files) in os.walk(root):
            fnames = []
            x86_found = False
            for f in files:
                path = os.path.join(directory_name, f)
                if os.path.isfile(path) and f.startswith(dll):
                    fnames.append(f)
                    if '_x86' in path:
                        x86_found = True

            for fname in fnames:
                fpath = os.path.join(directory_name, fname)
                if util.is_os_64bit():
                    if not cls.can_load_library(fpath):
                        continue
                    elif '_x86' not in fname:
                        yield fpath
                elif x86_found:
                    if '_x86' in fname:
                        yield fpath
                else:
                    yield fpath

    @classmethod
    def find_library_darwin(cls):
        """Loads the SEGGER DLL from the installed applications.

        This method accounts for the all the different ways in which the DLL
        may be installed depending on the version of the DLL.  Always uses
        the first directory found.

        SEGGER's DLL is installed in one of three ways dependent on which
        which version of the SEGGER tools are installed:

        ======== ============================================================
        Versions Directory
        ======== ============================================================
        < 5.0.0  ``/Applications/SEGGER/JLink\\ NUMBER``
        < 6.0.0  ``/Applications/SEGGER/JLink/libjlinkarm.major.minor.dylib``
        >= 6.0.0 ``/Applications/SEGGER/JLink/libjlinkarm``
        ======== ============================================================

        Args:
          cls (Library): the ``Library`` class

        Returns:
          The path to the J-Link library files in the order they are found.
        """
        dll = Library.JLINK_SDK_NAME
        root = os.path.join('/', 'Applications', 'SEGGER')
        if not os.path.isdir(root):
            return

        for d in os.listdir(root):
            dir_path = os.path.join(root, d)

            # Navigate through each JLink directory.
            if os.path.isdir(dir_path) and d.startswith('JLink'):
                files = list(f for f in os.listdir(dir_path) if
                             os.path.isfile(os.path.join(dir_path, f)))

                # For versions >= 6.0.0 and < 5.0.0, this file will exist, so
                # we want to use this one instead of the versioned one.
                if (dll + '.dylib') in files:
                    yield os.path.join(dir_path, dll + '.dylib')

                # For versions >= 5.0.0 and < 6.0.0, there is no strictly
                # linked library file, so try and find the versioned one.
                for f in files:
                    if f.startswith(dll):
                        yield os.path.join(dir_path, f)

    def __init__(self, dllpath=None, use_tmpcpy=None):
        """Initializes an instance of a ``Library``.

        Loads the default J-Link DLL if ``dllpath`` is ``None``, otherwise
        loads the DLL specified by the given ``dllpath``.

        Args:
          self (Library): the ``Library`` instance
          dllpath (str): the DLL to load into the library
          use_tmpcpy (Optional[bool]): ``True`` to load a temporary copy of
            J-Link DLL, ``None`` to dynamically decide based on DLL version.

        Returns:
          ``None``
        """
        self._lib = None
        self._winlib = None
        self._path = None
        self._windows = sys.platform.startswith('win')
        self._cygwin = sys.platform.startswith('cygwin')
        self._use_tmpcpy = use_tmpcpy
        self._temp = None

        if self._windows or self._cygwin:
            self._sdk = self.get_appropriate_windows_sdk_name()
        else:
            self._sdk = self.JLINK_SDK_OBJECT

        if dllpath is not None:
            self.load(dllpath)
        else:
            self.load_default()

    def __del__(self):
        """Cleans up the temporary DLL file created when the lib was loaded.

        Args:
          self (Library): the ``Library`` instance

        Returns:
          ``None``
        """
        self.unload()

    def load_default(self):
        """Loads the default J-Link SDK DLL.

        The default J-Link SDK is determined by first checking if ``ctypes``
        can find the DLL, then by searching the platform-specific paths.

        Args:
          self (Library): the ``Library`` instance

        Returns:
          ``True`` if the DLL was loaded, otherwise ``False``.
        """

        # Request the underlying operating system, through ctypes,
        # to resolve the J-Link DLL "the standard way" by its
        # library name.
        path = ctypes_util.find_library(self._sdk)

        # On Linux, find_library() actually returns the soname,
        # so we've got something like path = 'libjlinkarm.so.7',
        # and now have to retrieve the absolute file path.
        if (path is not None) and sys.platform.startswith('linux'):
            # For this, we'll rely on dlinfo(), which is not a POSIX API,
            # but a GNU libc extension.
            if platform.libc_ver()[0] == 'glibc':
                if Library._dlinfo is None:
                    Library._dlinfo = JLinkarmDlInfo(path)
                path = Library._dlinfo.path
            else:
                # When GNU libc extensions aren't available,
                # continue as if find_library() had failed.
                path = None

        if path is None:
            # Couldn't find it the standard way.  Fallback to the non-standard
            # way of finding the J-Link library.  These methods are operating
            # system specific.
            if self._windows or self._cygwin:
                path = next(self.find_library_windows(), None)
            elif sys.platform.startswith('linux'):
                path = next(self.find_library_linux(), None)
            elif sys.platform.startswith('darwin'):
                path = next(self.find_library_darwin(), None)

        if path is not None:
            return self.load(path)

        return False

    def load(self, path=None):
        """Loads the specified DLL, if any, otherwise re-loads the current DLL.

        If ``path`` is specified, loads the DLL at the given ``path``,
        otherwise re-loads the DLL currently specified by this library.

        Note:
          This creates a temporary DLL file to use for the instance.  This is
          necessary to work around a limitation of the J-Link DLL in which
          multiple J-Links cannot be accessed from the same process.

        Args:
          self (Library): the ``Library`` instance
          path (path): path to the DLL to load

        Returns:
          ``True`` if library was loaded successfully.

        Raises:
          OSError: if there is no J-LINK SDK DLL present at the path.

        See Also:
          `J-Link Multi-session <http://forum.segger.com/index.php?page=Thread&threadID=669>`_.
        """
        self.unload()

        self._path = path or self._path

        # Windows requires a proper suffix in order to load the library file,
        # so it must be set here.
        if self._windows or self._cygwin:
            suffix = '.dll'
        elif sys.platform.startswith('darwin'):
            suffix = '.dylib'
        else:
            suffix = '.so'

        lib_path = self._path

        if self._use_tmpcpy is None:
            # Fall back to ``True`` if version lookup failed (old pylink default)
            maybe_version = self.dll_version()
            self._use_tmpcpy = (maybe_version <= 6.10) if maybe_version else True

        if self._use_tmpcpy:
            # Copy the J-Link DLL to a temporary file.  This will be cleaned up the
            # next time we load a DLL using this library or if this library is
            # cleaned up.
            tf = tempfile.NamedTemporaryFile(delete=False, suffix=suffix)
            with open(tf.name, 'wb') as outputfile:
                with open(self._path, 'rb') as inputfile:
                    outputfile.write(inputfile.read())

            # This is needed to work around a WindowsError where the file is not
            # being properly cleaned up after exiting the with statement.
            tf.close()

            lib_path = tf.name
            self._temp = tf

        self._lib = ctypes.cdll.LoadLibrary(lib_path)

        if self._windows:
            # The J-Link library uses a mix of __cdecl and __stdcall function
            # calls.  While this is fine on a nix platform or in cygwin, this
            # causes issues with Windows, where it expects the __stdcall
            # methods to follow the standard calling convention.  As a result,
            # we have to convert them to windows function calls.
            self._winlib = ctypes.windll.LoadLibrary(lib_path)
            for stdcall in self._standard_calls_:
                if hasattr(self._winlib, stdcall):
                    # Backwards compatibility.  Some methods do not exist on
                    # older versions of the J-Link firmware, so ignore them in
                    # these cases.
                    setattr(self._lib, stdcall, getattr(self._winlib, stdcall))
        return True

    def unload(self):
        """Unloads the library's DLL if it has been loaded.

        This additionally cleans up the temporary DLL file that was created
        when the library was loaded.

        Args:
          self (Library): the ``Library`` instance

        Returns:
          ``True`` if the DLL was unloaded, otherwise ``False``.
        """
        unloaded = False

        if self._lib is not None:
            if self._winlib is not None:
                # ctypes passes integers as 32-bit C integer types, which will
                # truncate the value of a 64-bit pointer in 64-bit python, so
                # we have to change the FreeLibrary method to take a pointer
                # instead of an integer handle.
                ctypes.windll.kernel32.FreeLibrary.argtypes = (
                    ctypes.c_void_p,
                )

                # On Windows we must free both loaded libraries before the
                # temporary file can be cleaned up.
                ctypes.windll.kernel32.FreeLibrary(self._lib._handle)
                ctypes.windll.kernel32.FreeLibrary(self._winlib._handle)

                self._lib = None
                self._winlib = None

                unloaded = True
            else:
                # On OSX and Linux, just release the library; it's not safe
                # to close a dll that ctypes is using.
                del self._lib
                self._lib = None
                unloaded = True

        if self._temp is not None:
            os.remove(self._temp.name)
            self._temp = None

        return unloaded

    def dll(self):
        """Returns the DLL for the underlying shared library.

        Args:
          self (Library): the ``Library`` instance

        Returns:
          A ``ctypes`` DLL instance if one was loaded, otherwise ``None``.
        """
        return self._lib

    def dll_version(self):
        """Returns the DLL version number.

        Args:
          self (Library): the ``Library`` instance

        Returns:
            DLL version number (e.g. 7.96) or ``None`` if parsing fails.
        """
        if self._path is not None:
            match = re.search(r"JLink(?:_Linux)?_V(\d+)", self._path)
            if match:
                num_part = match.group(1)
                return float(f"{num_part[0]}.{num_part[1:]}")
        return None


class JLinkarmDlInfo:
    """Helper to retrieve the absolute path of the JLink library (aka DLL)
    based on its soname.

    This is used on Linux, where ctypes.util.find_library() will not return
    the library full file path, but only the file name (aka soname).
    We'll then rely on the native dlinfo() API to retrieve the library absolute
    file path.

    For e.g.:
    - LD_LIBRARY_PATH=/mnt/platform/segger/JLink
    - ctypes.util.find_library('jlinkarm') -> libjlinkarm.so.7
    - JLinkarmDlInfo('libjlinkarm.so.7').path -> /mnt/platform/segger/JLink/libjlinkarm.so.7

    Note that unlike dlopen(), dlsym() and friends, dlinfo() is not a POSIX API,
    but a GNU extension, available only on systems with the GNU libc implementation (glibc).

    The dlinfo() dance implementation is adapted from @cloudflightio,
    https://github.com/cloudflightio/python-dlinfo.
    """

    # Request to obtain a pointer to the link_map structure corresponding
    # to a given handle (Linux).
    # See: man dlinfo(3)
    RTLD_DI_LINKMAP = 2

    # dlinfo(3): struct link_map, where l_name will be the file path.
    class LinkMap(ctypes.Structure):
        """
        Represents a C struct link_map (Linux).
        See: man dlinfo(3)
        """
        _fields_ = [
            ('l_addr', ctypes.c_void_p),
            ('l_name', ctypes.c_char_p),
            ('l_ld', ctypes.c_void_p),
            ('l_next', ctypes.c_void_p),
            ('l_prev', ctypes.c_void_p),
        ]

    def __init__(self, jlinkarm_soname):
        """Retrieves the absolute file path of the JLink library (aka DLL)
        corresponding to the given soname.

        This runs the dlinfo() dance using the ctypes API:
        - loads the JLink DLL shared object for given soname
        - loads the dl library and lookup the dlinfo symbol
        - calls dlinfo() with request RTLD_DI_LINKMAP to get the struct link_map
          for the JLink library
        - access the struct content to retrieve the library's absolute path

        The dlinfo() dance implementation does not try to hide any OSError
        the ctypes API may raise, since this would likely hide a host
        system configuration issue (aka "should not happen").

        Args:
        - jlinkarm_soname: The JLink DLL soname returned by find_library(),
          for e.g. 'libjlinkarm.so.7'.

        Raises:
          - OSError when the actual library file has been removed, is not readable,
            is not a loadable shared object, etc.
        """
        # JLink DLL's absolute file path.
        self._dll_path = None

        # We're using the soname returned by a successful call to find_library(),
        # hence we can expect LoadLibrary() to in turn successfully load the JLink DLL.
        tmp_cdll_jlink = ctypes.cdll.LoadLibrary(jlinkarm_soname)

        # dlinfo() dance to retrieve the library's absolute file path.
        #
        # This code path should be involved only for POSIX systems
        # with GNU libc, where:
        # - libdl is available (POSIX)
        # - dlinfo() is defined (glibc)
        #
        # Hence bellow calls to find_library(), LoadLibrary(), and dlinfo() should succeed.
        dl_soname = ctypes_util.find_library('dl')
        if dl_soname is not None:
            tmp_cdll_dl = ctypes.cdll.LoadLibrary(dl_soname)
            dlinfo = tmp_cdll_dl.dlinfo
            dlinfo.argtypes = ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p
            dlinfo.restype = ctypes.c_int

            linkmap = ctypes.c_void_p()
            if dlinfo(tmp_cdll_jlink._handle, JLinkarmDlInfo.RTLD_DI_LINKMAP, ctypes.byref(linkmap)) == 0:
                linkmap = ctypes.cast(linkmap, ctypes.POINTER(JLinkarmDlInfo.LinkMap))
                self._dll_path = linkmap.contents.l_name.decode(sys.getdefaultencoding())

            # "Free" tmp dl library
            del tmp_cdll_dl
            tmp_cdll_dl = None

        else:
            # Should not happen.
            pass

        # "Free" tmp jlinkarm library
        del tmp_cdll_jlink
        tmp_cdll_jlink = None

    @property
    def path(self):
        """Answers the JLink DLL file path.

        Returns the JLink DLL's absolute path,
        or None when the dl library is unavailable despite the system
        presenting itself as POSIX.
        """
        return self._dll_path



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import behave


@behave.given('I invalidate the firmware')
def step_firmware_invalidate(context):
    """Invalidtes the emulator's firmware.

    Args:
      context: the ``Context`` instance

    Returns:
      ``None``
    """
    context.jlink.invalidate_firmware()


@behave.then('I can update the firmware')
def step_firmware_update(context):
    """Asserts that the firmware can be updated.

    Args:
      context: the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.update_firmware() >= 0


@behave.then('I can force a firmware update')
def step_force_firmware_update(context):
    """Asserts that the firmware can be force updated.

    Args:
      context: the ``Context`` instance

    Returns:
      ``None``
    """
    assert context.jlink.update_firmware() >= 0

    log_messages = context.log.getvalue().split('\n')
    assert 'New firmware booted successfully' in log_messages[-1]



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading


class ThreadReturn(threading.Thread):
    """Implementation of a thread with a return value.

    See also:
      `StackOverflow <http://stackoverflow.com/questions/6893968/>`__.
    """

    def __init__(self, daemon=False, *args, **kwargs):
        """Initializes the thread.

        Args:
          self (ThreadReturn): the ``ThreadReturn`` instance
          daemon (bool): if the thread should be spawned as a daemon
          args: optional list of arguments
          kwargs: optional key-word arguments

        Returns:
          ``None``
        """
        super(ThreadReturn, self).__init__(*args, **kwargs)
        self.daemon = daemon
        self._return = None

    def run(self):
        """Runs the thread.

        Args:
          self (ThreadReturn): the ``ThreadReturn`` instance

        Returns:
          ``None``
        """
        target = getattr(self, '_Thread__target', getattr(self, '_target', None))
        args = getattr(self, '_Thread__args', getattr(self, '_args', None))
        kwargs = getattr(self, '_Thread__kwargs', getattr(self, '_kwargs', None))
        if target is not None:
            self._return = target(*args, **kwargs)

        return None

    def join(self, *args, **kwargs):
        """Joins the thread.

        Args:
          self (ThreadReturn): the ``ThreadReturn`` instance
          args: optional list of arguments
          kwargs: optional key-word arguments

        Returns:
          The return value of the exited thread.
        """
        super(ThreadReturn, self).join(*args, **kwargs)
        return self._return
